import base64
import logging

from python_sdk_client.cropin_api import CropinAPI

"""Example on how to use CropAPI.  
"""
if __name__ == '__main__':
    logging.info(">>>>>>>>>>>> starting")

    # authenticate
    cropin_api = CropinAPI("test", "12121212", "password")

    # plot details without any filter
    plot_response = cropin_api.get_plot_details()
    print("plot details without any filters is : {}".format(plot_response))

    # plot details with ids passed
    plot_response = cropin_api.get_plot_details(plot_ids='62305d0110197fce8a2d7b40, 62c550ba793e6d7ad6e122d1')
    print("plot details when plot ids are passed : {}".format(plot_response))

    # plot details with externalId passed
    plot_response = cropin_api.get_plot_details(external_ids='8104202')
    print("plot details when external Id is passed: {}".format(plot_response))

    # plot details with plot_ids and external_ids passed
    plot_response = cropin_api.get_plot_details(plot_ids='62305d0110197fce8a2d7b40, 62c550ba793e6d7ad6e122d1',
                                                external_ids='8104202')
    print("plot details when ids and external Id is passed: {}".format(plot_response))

    # satellite data without any filter
    satellite_response = cropin_api.get_satellite_details()
    print("satellite metrics with no filters :{}".format(satellite_response))

    # satellite data with boundary_id filter
    satellite_response = cropin_api.get_satellite_details(boundary_ids='61e8ec4c09e1880458676b35')
    print("satellite metrics when boundaryId is passed : {}".format(satellite_response))

    # yield data without any filter
    yield_response = cropin_api.get_yield_details()
    print("yield data with no filter passed :{}".format(yield_response))

    # weather data without any filter
    weather_response = cropin_api.get_weather_details()
    print("weather data with no filter passed :{}".format(weather_response))

    # plot image download
    download_response = cropin_api.download_image(plot_id='62c557436077c155e3b74ff4', image_name='NDVI',
                                                  image_type='PNG', date='2022-05-24')
    print("download_response is: {}".format(download_response))

    # code to convert the bytes to image
    imgdata = base64.b64decode(download_response['bytes'])
    filename = 'image.jpg'
    with open(filename, 'wb') as f:
        f.write(imgdata)



class InsightServiceCfg:
    PROD_BASE_URL = "https://t608jbcx83.execute-api.ap-southeast-1.amazonaws.com/dev/api/v1"
    QA_BASE_URL = "https://rn0t0s00e2.execute-api.ap-southeast-1.amazonaws.com/qa/api/v1"
    X_API_KEY = "X-Api-Key"
    BATCH_SIZE = 300
    MAX_SIZE = 5000





import logging

from python_sdk_client.clients_enum import EnvType
from abc import ABC

from python_sdk_client.libs.keycloak_client import KeyCloakAPI


class AbstractClient(ABC):

    def __init__(self, tenant: str, username: str, password: str, env: EnvType) -> None:
        self.tenant = tenant
        self.env = env
        self.org_id = 'sf_plus_' + str(tenant)
        self.token, self.refresh_token, self.x_api_key = KeyCloakAPI(env).authenticate(tenant, username, password)
        logging.info("Super class is initialized")

    def get_auth(self) -> str:
        return self.token

    def refresh_auth(self) -> (str, str):
        self.token, self.refresh_token = KeyCloakAPI.re_authenticate(self.refresh_token())



import logging

from insights_python_client import Configuration
from python_sdk_client.clients_enum import EnvType

from python_sdk_client.libs.client_cfg import InsightServiceCfg
from python_sdk_client.libs.cropin_exceptions import UnauthorizedError

"""

Keycloak Client
----------------

A class for authenticating the user

"""


class KeyCloakAPI:
    """
    Generates token given tenant, username and password as input
    """
    logging = logging.getLogger('KeyCloakAPI')

    def __init__(self, env: EnvType):
        self.env = env


    def authenticate(self, tenant: str, username: str, password: str) -> (str, str):
        logging.info("Starting authentication now")
        token = None
        refresh_token = None
        x_api_key = None
        response = self._internal_sdk_auth_check(tenant, username, password)
        if response is not None:
            token = response.to_dict().get('access_token')
            refresh_token = response.to_dict().get('refresh_token')
            x_api_key = response.to_dict().get('x_api_key')
        if token is None:
            raise UnauthorizedError("Authentication not successful token is None")
        if refresh_token is None:
            raise UnauthorizedError("Authentication not successful refresh_token is None")
        if x_api_key is None:
            raise UnauthorizedError("Authentication not successful x_api_key is None")
        logging.info("Authentication successful! Ready to make function call.")
        return token, refresh_token, x_api_key

    """
    Get access token using the refresh token for authentication
    """

    def re_authenticate(self, refresh_token: str) -> (str, str):
        token = None
        refresh_token = None
        x_api_key = None
        # TODO: refresh token using SDK
        if token is None or refresh_token is None:
            raise UnauthorizedError("Authentication not successful")
        return token, refresh_token, x_api_key

    """
    Validating the returned token
    """

    def validate_token(self, token: str) -> bool:
        is_valid = True
        # TODO: check if token has expired
        return is_valid

    def _internal_sdk_auth_check(self, tenant: str, username: str, password: str):
        import insights_python_client
        from insights_python_client.rest import ApiException

        configuration = Configuration()
        if self.env == EnvType.QA:
            configuration.host = InsightServiceCfg.QA_BASE_URL
        elif self.env == EnvType.PROD:
            configuration.host = InsightServiceCfg.PROD_BASE_URL
        else:
            configuration.host = InsightServiceCfg.QA_BASE_URL

        api_instance = insights_python_client.TokenApi(insights_python_client.ApiClient(configuration))
        body = insights_python_client.AccessTokenRequest(tenant=tenant, username=username,
                                                         password=password)  # AccessTokenRequest |
        try:
            # Generate access token
            logging.info("doing authentication for request: {}".format(body))
            api_response = api_instance.get_token(body)
            logging.info("auth api response: {0}".format(api_response))
            return api_response
        except ApiException as e:
            logging.error("Exception when calling AuthenticateApi->get_token: {}\n".format(e))
            raise Exception("failed to auth ", e)


import insights_python_client
from insights_python_client import Configuration

from python_sdk_client.libs.abstract_client import AbstractClient
from python_sdk_client.libs.batch_processor import handle
from python_sdk_client.libs.client_cfg import InsightServiceCfg
from python_sdk_client.clients_enum import EnvType

"""
Insights Service Client
-----------------------

class to validate the inputs and set the env, endpoint and other env specific details.
"""


class InsightServiceClient(AbstractClient):
    """
    Initialising the env and base url
    """

    def __init__(self, tenant: str, username: str, password: str, env: EnvType) -> None:
        super(InsightServiceClient, self).__init__(tenant, username, password, env)

        self.tenant_type = 'SMARTFARM_PLUS'
        if env == EnvType.PROD:
            self.base_url = InsightServiceCfg.PROD_BASE_URL
        elif env == EnvType.QA:
            self.base_url = InsightServiceCfg.QA_BASE_URL

        self.configuration = Configuration()
        # set base url
        self.configuration.host = self.base_url
        # set auth token
        self.configuration.api_key['Authorization'] = self.token

    """
    Validate input for fetching plot details
    """

    def get_plot_details(self, plot_ids: str, **kwargs):

        boundary_api = insights_python_client.PlotApi(insights_python_client.ApiClient(self.configuration))
        plot_ids_resp = handle(boundary_api.list_all3, plot_ids, self.tenant_type, self.x_api_key, self.org_id,
                               batch_size=InsightServiceCfg.BATCH_SIZE, **kwargs)
        return plot_ids_resp

    """
    Validate inputs for satellite details
    """

    def get_satellite_details(self, plot_ids: str, **kwargs):

        metrics_api = insights_python_client.MetricsApi(insights_python_client.ApiClient(self.configuration))
        plot_ids_resp = handle(metrics_api.list_all2, plot_ids, self.tenant_type, self.x_api_key, self.org_id,
                               batch_size=InsightServiceCfg.BATCH_SIZE, **kwargs)
        return plot_ids_resp

    """
    Validate inputs for weather details
    """

    def get_weather_details(self, plot_ids: str, **kwargs):

        weather_api = insights_python_client.WeatherApi(insights_python_client.ApiClient(self.configuration))
        weather_api_resp = handle(weather_api.list_all1, plot_ids, self.tenant_type, self.x_api_key, self.org_id,
                                  batch_size=InsightServiceCfg.BATCH_SIZE, **kwargs)
        return weather_api_resp

    """ 
    Validate inputs for yield details
    """

    def get_yield_details(self, plot_ids: str, **kwargs):
        yield_api = insights_python_client.YieldApi(insights_python_client.ApiClient(self.configuration))
        yield_api_resp = handle(yield_api.list_all, plot_ids, self.tenant_type, self.x_api_key, self.org_id,
                                batch_size=InsightServiceCfg.BATCH_SIZE, **kwargs)
        return yield_api_resp

    """
    Validate inputs for download plot image
    """

    def download_image(self, plot_id: str, image_name, image_type, date):
        download_api = insights_python_client.FileApi(insights_python_client.ApiClient(self.configuration))
        file_response = download_api.get_plot_image_for_satellite_and_health_indices(self.tenant_type, self.x_api_key,
                                                                                     org_id=self.org_id,
                                                                                     image_type=image_type,
                                                                                     image_name=image_name, _date=date,
                                                                                     boundary_id=plot_id)
        return file_response



import logging

from python_sdk_client.clients_enum import EnvType
from python_sdk_client.libs.insight_service_client import InsightServiceClient


class CropinClient:

    def __init__(self, tenant: str, username: str, password: str, env: EnvType):
        logging.info("Initializing Cropin Client")
        self.insight_service_client = InsightServiceClient(tenant, username, password, env)
        logging.info("Initialized Cropin Client")

    def get_plot_details(self, plot_ids: str, **kwargs):
        return self.insight_service_client.get_plot_details(plot_ids, **kwargs)

    def get_satellite_details(self, plot_ids: str, **kwargs):
        return self.insight_service_client.get_satellite_details(plot_ids, **kwargs)

    def get_weather_details(self, plot_ids: str, **kwargs):
        return self.insight_service_client.get_weather_details(plot_ids, **kwargs)

    def get_yield_details(self, plot_ids: str, **kwargs):
        return self.insight_service_client.get_yield_details(plot_ids, **kwargs)

    def download_image(self, plot_id: str, image_name, image_type, date):
        return self.insight_service_client.download_image(plot_id, image_name, image_type, date)



import logging

from python_sdk_client.clients_enum import EnvType
from python_sdk_client.libs.cropin_client import CropinClient


class CropinAPI:
    """Class to connect CropinAPI to do different operations.
    As part of initialization, it's first step is Authentication.

    Parameters
    ----------
    tenant : string
        tenant for Cropin
    username : string
        username for Cropin
    password : string
        password for Cropin

    """

    logger = logging.getLogger("insights-sdk.CropinAPI")

    def __init__(self, tenant: str, username: str, password: str, env: EnvType = EnvType.PROD):
        self.cropin_client = CropinClient(tenant, username, password, env)
        print("CropinAPI is ready !!!!")

    def get_plot_details(self, plot_ids: str = None, **kwargs) -> list:
        """
        Fetch plot's details by providing multiple plot_ids as list.
        If list of ids is not provided, it will return few latest plot's details.

        Parameters
        ----------
        :param str plot_ids: Filter by ids, it can be single id or list
        :param str external_ids: Filter by externalIds
        :param str parent_ids: Filter by parentIds
        :param str min_created_date_time: Filter by specific date range
        :param str max_created_date_time: Filter by specific date range
        :param str page: Page no.
        :param str size: Page size
        :param str sort_by: Sort by given parameter name
        :param str direction: Order by ascending or descending fashion

        :return: List[Plots]

        """
        return self.cropin_client.get_plot_details(plot_ids, **kwargs)

    def get_satellite_details(self, ids: str = None, **kwargs) -> list:
        """ Fetch satellite details for the plot ids passed.

        Satellite details returns data for satellite indices and
        crop details for the models subscribed.

        :param str ids: Filter by ids, it can be single id or list
        :param str boundary_id: Filter by boundaryId
        :param str captured_date_time: Filter by capturedDateTime
        :param str min_cloud_coverage: Filters by cloudCoverge >= minCloudCoverage
        :param str max_cloud_coverage: Filters by cloudCoverge <= maxCloudCoverage
        :param str page: Page no.
        :param str size: Page size
        :param str sort_by: Sort by given parameter name
        :param str direction: Order by ascending or descending fashion


        :return: List[SatelliteMetricsResponse]
        """
        return self.cropin_client.get_satellite_details(ids, **kwargs)

    def get_weather_details(self, ids: str = None, **kwargs) -> list:
        """
        Fetches weather details for the plot ids passed.

        :param str ids: Filter by ids, it can be single id or list
        :param str plot_id: Filter by plotId
        :param str _date: Filter by date
        :param str date_to: Filter by specific date range
        :param str date_from: Filter by specific date range
        :param str external_id: Filter by externalId
        :param str page: Page no.
        :param str size: Page size
        :param str sort_by: Sort by given parameter name
        :param str direction: Order by ascending or descending fashion

        :return: List[WeatherDataResponse]
        """
        return self.cropin_client.get_weather_details(ids, **kwargs)

    def get_yield_details(self, ids: str = None, **kwargs) -> list:
        """
        Fetches yield details for the plot ids passed.

        :param str ids: Filter by ids, it can be single id or list
        :param str boundary_id: Filter by boundaryId
        :param str external_id: Filter by externalId
        :param str page: Page no.
        :param str size: Page size
        :param str sort_by: Sort by given parameter name
        :param str direction: Order by ascending or descending fashion
        :return: List[YieldResponse]

        """
        return self.cropin_client.get_yield_details(ids, **kwargs)

    def download_image(self, plot_id: str, image_name, image_type, date) -> dict:
        """

        Returns plot image byte streams for a single plot id at a given date and image type and name.

        :param str boundary_id:  Boundary Id (required)
        :param str _date: Date (required)
        :param str image_name: Plot image name (required)
        :param str org_id: orgId
        :param str image_type: Plot image type
        :param str tile_size: Tile size , default value - [255,255]

        :return: PlotImageResponse

        """
        return self.cropin_client.download_image(plot_id, image_name, image_type, date)



import numbers

from python_sdk_client.libs.client_cfg import InsightServiceCfg


def handle(anotherfunc, input_str, tenant_type: str, x_api_key: str, org_id: str, batch_size: numbers = 300, **kwargs):
    response = []
    if input_str is None:
        temp_resp = anotherfunc(tenant_type, x_api_key, org_id=org_id, **kwargs)
        if temp_resp is None or 'records' not in temp_resp or temp_resp['records'] is None:
            return response

        response = response + temp_resp['records']
        if temp_resp['totalPages'] is not None:
            total_pages = temp_resp['totalPages']
            if temp_resp['currentPageItems'] > 0:
                total_pages = min(total_pages, InsightServiceCfg.MAX_SIZE / temp_resp['currentPageItems'])
            for i in range(1, total_pages, 1):
                temp_resp = anotherfunc(tenant_type, x_api_key, org_id=org_id, page=i, *kwargs)
                response = response + temp_resp['records']
        return response

    if input_str is not None:
        input_list = input_str.split(", ")
        for i in range(0, len(input_list), batch_size):
            current_batch = input_list[i:i + batch_size]
            current_batch_str = ','.join(current_batch)
            temp_resp = anotherfunc(tenant_type, x_api_key, org_id=org_id, ids=current_batch_str, **kwargs)
            if temp_resp is not None and 'records' in temp_resp and temp_resp['records'] is not None:
                response = response + temp_resp['records']
        return response



from setuptools import setup, find_packages  # noqa: H301

NAME = "cropin-plotrisk-insight-client"
VERSION = "0.0.6"
# To install the library, run the following
#
# python setup.py install
#
# prerequisite: setuptools
# http://pypi.python.org/pypi/setuptools

REQUIRES = ["urllib3 >= 1.15", "six >= 1.10", "certifi", "python-dateutil", "cropin-insights-client >= 0.0.3"]

setup(
    name=NAME,
    version=VERSION,
    description="Insights Service API",
    author_email="",
    url="",
    keywords=["Swagger", "Insights Service API -test"],
    install_requires=REQUIRES,
    packages=find_packages(),
    include_package_data=True,
    long_description="""\
    ## Introduction  Swaggerâ¢ is a project used to describe and document RESTful APIs. This section provides an overview of the API design, methods, and supported use cases.    Most of the endpoints accept **JSON** as input and return **JSON** responses.  &#x60;&#x60;&#x60;  Content-type: application/json   Accept: application/json  &#x60;&#x60;&#x60;  ## Resources  A resource in Swagger is an entity that has a set of exposed operations. The entity can represent an actual object (account, users..) or a set of logical operations collated together. It is up to the specification user to decide whether sub-resources should be referred to as part of their main resource or as a resource of their own. For example, assume the following URL set:     &#x60;&#x60;&#x60;    /users         -GET                   POST                    /users/{id}    -GET                   PATCH                   DELETE   &#x60;&#x60;&#x60;    ## CRUD Operations  The platform supports Create, Read, Update, and Delete operations on most resources. You can review the standards for these operations and their standard parameters below.    Some endpoints have special behavior as exceptions.    ### Create  To create a resource, you typically submit an HTTP POST request with the resource&#x27;s required metadata in the request body. The response returns a 201 Created response code upon success with the resource&#x27;s metadata, including its internal id, in the response body.    ### Read  The HTTP GET request can be used to read a resource or to list a number of resources.    A resource&#x27;s id can be submitted in the request parameters to read a specific resource. The response usually returns a 200 OK response code upon success, with the resource&#x27;s metadata in the response body.    If a GET request does not include a specific resource id, it is treated as a list request. The response usually returns a 200 OK response code upon success, with an object containing a list of resources&#x27; metadata in the response body.    When reading resources, some common query parameters are usually available. e.g. :  &#x60;&#x60;&#x60;  v1/users?size&#x3D;25&amp;page&#x3D;1  &#x60;&#x60;&#x60;  **Query Parameter Type Description**  - size should be between 10 and 300 (default value 50)    - page should be a zero or any positive number (default value 0)    ### Update  Updating a resource requires the resource id, and is typically done using an HTTP PATCH request, with the fields to modify in the request body. The response usually returns a 200 OK response code upon success, with information about the modified resource in the response body.    ### Delete  Deleting a resource requires the resource id and is typically executing via an HTTP DELETE request. The response usually returns a 204 No Content response code upon success.    ## Trying the API  You can use [Swagger UI](https://swagger.io/tools/swagger-ui/) or any third party client such as ,[Postman](https://www.postman.com/), [cURL](https://curl.se/) etc. to test the REST API.    **Swagger UI**    - Authorize API by clicking on **Authorize** button.   - Select an API resource  - Click on **Try it out**   - filterQuery (Parameters): If required add parameters else keep blank json  &#x60;&#x60;&#x60;   e.g.   {           \&quot;size\&quot;: 300,           \&quot;page\&quot;: 0,          }             e.g.   {          }            &#x60;&#x60;&#x60;              - Fill the headers like TenantType, OrgId &amp; X-Api-Key   - Click on **Execute**    **Curl**  &#x60;&#x60;&#x60;  curl -X &#x27;GET&#x27; \\  &#x27;http://localhost:8085/api/v1/crops&#x27; \\  -H &#x27;accept: application/json&#x27; \\  -H &#x27;TenantType: SMARTRISK&#x27; \\  -H &#x27;orgId: test&#x27; \\  -H &#x27;X-Api-Key: qwjokooopppp&#x27; \\  -H &#x27;Authorization: Bearer jklkopalkddlplplllllllllll2340k&#x27;  &#x60;&#x60;&#x60;  Using a graphical tool such as Postman, it is possible to import the API specifications directly:    - Download the API specification by clicking the **Download** button at top of this document  - Import the JSON specification in the graphical tool of your choice.  - In *Postman*, you can click the import button at the top    ## Enabling CORS  [Cross-origin resource sharing (CORS)](https://developer.mozilla.org/en-US/docs/Web/HTTP/CORS) is a browser security feature that restricts HTTP requests that are initiated from scripts running in the browser.      ## Authentication  The API authentication process validates the identity of the client attempting to make a connection by using an authentication protocol. REST API support many authentication methods which are as follows :    **HTTP Authentication Schemes**:    - Basic  - Bearer  - Digest  - OAuth and others...    Insights Service uses Bearer Authentication (also called token authentication) is an HTTP authentication scheme that involves security tokens called bearer tokens.    **Access Token** can be generated by following API by passing valid request payload (see Authenticate API)  &#x60;&#x60;&#x60;  POST   /api/v1/authenticate  &#x60;&#x60;&#x60;    For more details on authentication, see [API Authentication](https://www.3pillarglobal.com/insights/most-popular-api-authentication-methods).    ## Errors  The REST API reports errors by returning an appropriate HTTP response code, for example 404 (Not Found), and a JSON response. Any HTTP response code that is not in the range 200 - 299 is considered an error.    ### BadRequest (400)  This response means that the server cannot or will not process the request due to something that is perceived to be a client error (e.g., malformed request syntax, invalid request message framing, or deceptive request routing). To resolve this, please ensure that your syntax is correct.  ### Unauthorized (401)  Indicates that the request requires user authentication information. The client may repeat the request with a suitable Authorization header field.    ### Forbidden (403)  Unauthorized request. The client does not have access rights to the content. Unlike 401, the clientâs identity is known to the server.    ### NotFound (404)  The server can not find the requested resource.    ### MethodNotAllowed (405)  The request HTTP method is known by the server but has been disabled and cannot be used for that resource.  ### NotAcceptable (406)  The server does not find any content that conforms to the criteria given by the user agent in header sent in the request.  ### AlreadyExists  The request could not be completed due to a conflict with the current state of the target resource, e.g. the resource it tries to create already exists.  ### Internal Server Error (500)  The server encountered an unexpected condition that prevented it from fulfilling the request.    For more details on HTTP Status code see [HTTP Status Codes](https://restfulapi.net/http-status-codes/)   # noqa: E501
    """
)
